from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.group_create_group_body_metadata import GroupCreateGroupBodyMetadata
from ..models.visibility_level import VisibilityLevel
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="GroupCreateGroupBody")


@_attrs_define
class GroupCreateGroupBody:
    """GroupCreateGroupBody model

    Attributes:
        name (str):
        controlled_access (Union[Unset, bool]):  Default: False.
        metadata (Union[Unset, GroupCreateGroupBodyMetadata]):
        visibility_level (Union[Unset, VisibilityLevel]):
    """

    name: str
    controlled_access: Union[Unset, bool] = False
    metadata: Union[Unset, "GroupCreateGroupBodyMetadata"] = UNSET
    visibility_level: Union[Unset, VisibilityLevel] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        name = self.name
        controlled_access = self.controlled_access
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()
        visibility_level: Union[Unset, str] = UNSET
        if not isinstance(self.visibility_level, Unset):
            visibility_level = self.visibility_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if controlled_access is not UNSET:
            field_dict["controlledAccess"] = controlled_access
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if visibility_level is not UNSET:
            field_dict["visibilityLevel"] = visibility_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`GroupCreateGroupBody` from a dict"""
        d = src_dict.copy()
        name = d.pop("name")

        controlled_access = d.pop("controlledAccess", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, GroupCreateGroupBodyMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = GroupCreateGroupBodyMetadata.from_dict(_metadata)

        _visibility_level = d.pop("visibilityLevel", UNSET)
        visibility_level: Union[Unset, VisibilityLevel]
        if isinstance(_visibility_level, Unset):
            visibility_level = UNSET
        else:
            visibility_level = VisibilityLevel(_visibility_level)

        group_create_group_body = cls(
            name=name,
            controlled_access=controlled_access,
            metadata=metadata,
            visibility_level=visibility_level,
        )

        return group_create_group_body



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.user import User


T = TypeVar("T", bound="GroupMembersUsers")


@_attrs_define
class GroupMembersUsers:
    """GroupMembersUsers model"""

    additional_properties: Dict[str, "User"] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`GroupMembersUsers` from a dict"""
        d = src_dict.copy()
        group_members_users = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = User.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        group_members_users.additional_properties = additional_properties
        return group_members_users

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "User":
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: "User") -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.collection import Collection
from ..models.highlights import Highlights
from ..models.search_collections_response_hits_item_data_elements_item import (
    SearchCollectionsResponseHitsItemDataElementsItem,
)
from ..models.search_collections_response_hits_item_items_item import (
    SearchCollectionsResponseHitsItemItemsItem,
)


T = TypeVar("T", bound="SearchCollectionsResponseHitsItem")


@_attrs_define
class SearchCollectionsResponseHitsItem:
    """SearchCollectionsResponseHitsItem model

    Attributes:
        collection (Collection): Core collection properties, with IDs
        data_elements (List['SearchCollectionsResponseHitsItemDataElementsItem']):
        highlights (Highlights):
        items (List['SearchCollectionsResponseHitsItemItemsItem']):
        score (float):
    """

    collection: "Collection"
    data_elements: List["SearchCollectionsResponseHitsItemDataElementsItem"]
    highlights: "Highlights"
    items: List["SearchCollectionsResponseHitsItemItemsItem"]
    score: float

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        collection = self.collection.to_dict()
        data_elements = []
        for data_elements_item_data in self.data_elements:
            data_elements_item = data_elements_item_data.to_dict()
            data_elements.append(data_elements_item)

        highlights = self.highlights.to_dict()
        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()
            items.append(items_item)

        score = self.score

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "collection": collection,
                "dataElements": data_elements,
                "highlights": highlights,
                "items": items,
                "score": score,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchCollectionsResponseHitsItem` from a dict"""
        d = src_dict.copy()
        collection = Collection.from_dict(d.pop("collection"))

        data_elements = []
        _data_elements = d.pop("dataElements")
        for data_elements_item_data in _data_elements:
            data_elements_item = (
                SearchCollectionsResponseHitsItemDataElementsItem.from_dict(
                    data_elements_item_data
                )
            )

            data_elements.append(data_elements_item)

        highlights = Highlights.from_dict(d.pop("highlights"))

        items = []
        _items = d.pop("items")
        for items_item_data in _items:
            items_item = SearchCollectionsResponseHitsItemItemsItem.from_dict(
                items_item_data
            )

            items.append(items_item)

        score = d.pop("score")

        search_collections_response_hits_item = cls(
            collection=collection,
            data_elements=data_elements,
            highlights=highlights,
            items=items,
            score=score,
        )

        return search_collections_response_hits_item



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.collections_items_complete_multipart_upload_body_action import (
    CollectionsItemsCompleteMultipartUploadBodyAction,
)
from ..models.collections_items_complete_multipart_upload_body_checksum import (
    CollectionsItemsCompleteMultipartUploadBodyChecksum,
)
from ..models.collections_items_complete_multipart_upload_body_parts_item import (
    CollectionsItemsCompleteMultipartUploadBodyPartsItem,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionsItemsCompleteMultipartUploadBody")


@_attrs_define
class CollectionsItemsCompleteMultipartUploadBody:
    """CollectionsItemsCompleteMultipartUploadBody model

    Attributes:
        parts (List['CollectionsItemsCompleteMultipartUploadBodyPartsItem']):
        upload_id (str):
        upload_key (str):
        url (str):
        action (Union[Unset, CollectionsItemsCompleteMultipartUploadBodyAction]):
        checksum (Union[Unset, CollectionsItemsCompleteMultipartUploadBodyChecksum]):
    """

    parts: List["CollectionsItemsCompleteMultipartUploadBodyPartsItem"]
    upload_id: str
    upload_key: str
    url: str
    action: Union[Unset, CollectionsItemsCompleteMultipartUploadBodyAction] = UNSET
    checksum: Union[
        Unset, "CollectionsItemsCompleteMultipartUploadBodyChecksum"
    ] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        parts = []
        for parts_item_data in self.parts:
            parts_item = parts_item_data.to_dict()
            parts.append(parts_item)

        upload_id = self.upload_id
        upload_key = self.upload_key
        url = self.url
        action: Union[Unset, str] = UNSET
        if not isinstance(self.action, Unset):
            action = self.action.value

        checksum: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.checksum, Unset):
            checksum = self.checksum.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "parts": parts,
                "uploadId": upload_id,
                "uploadKey": upload_key,
                "url": url,
            }
        )
        if action is not UNSET:
            field_dict["action"] = action
        if checksum is not UNSET:
            field_dict["checksum"] = checksum

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsItemsCompleteMultipartUploadBody` from a dict"""
        d = src_dict.copy()
        parts = []
        _parts = d.pop("parts")
        for parts_item_data in _parts:
            parts_item = CollectionsItemsCompleteMultipartUploadBodyPartsItem.from_dict(
                parts_item_data
            )

            parts.append(parts_item)

        upload_id = d.pop("uploadId")

        upload_key = d.pop("uploadKey")

        url = d.pop("url")

        _action = d.pop("action", UNSET)
        action: Union[Unset, CollectionsItemsCompleteMultipartUploadBodyAction]
        if isinstance(_action, Unset):
            action = UNSET
        else:
            action = CollectionsItemsCompleteMultipartUploadBodyAction(_action)

        _checksum = d.pop("checksum", UNSET)
        checksum: Union[Unset, CollectionsItemsCompleteMultipartUploadBodyChecksum]
        if isinstance(_checksum, Unset):
            checksum = UNSET
        else:
            checksum = CollectionsItemsCompleteMultipartUploadBodyChecksum.from_dict(
                _checksum
            )

        collections_items_complete_multipart_upload_body = cls(
            parts=parts,
            upload_id=upload_id,
            upload_key=upload_key,
            url=url,
            action=action,
            checksum=checksum,
        )

        return collections_items_complete_multipart_upload_body



"""List collections"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import List
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.collection_access_level import CollectionAccessLevel
from ...models.collections_get_collections_collection_list import (
    CollectionsGetCollectionsCollectionList,
)
from ...models.collections_get_collections_sort import CollectionsGetCollectionsSort
from ...models.collections_get_collections_sort_dir import (
    CollectionsGetCollectionsSortDir,
)
from ...models.collections_get_collections_version import (
    CollectionsGetCollectionsVersion,
)
from ...models.server_error import ServerError
from ...types import Response
from ...types import UNSET
from ...types import Unset


def _get_kwargs(
    *,
    id: Union[Unset, str] = UNSET,
    id_prefix: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    account_id: Union[Unset, str] = UNSET,
    not_account_id: Union[Unset, str] = UNSET,
    meta: Union[Unset, str] = UNSET,
    bmeta: Union[Unset, str] = UNSET,
    version: Union[Unset, CollectionsGetCollectionsVersion] = UNSET,
    version_suffix: Union[Unset, str] = UNSET,
    public: Union[Unset, bool] = UNSET,
    shared_with: Union[Unset, str] = UNSET,
    editable: Union[Unset, bool] = UNSET,
    starred: Union[Unset, bool] = UNSET,
    access_level: Union[Unset, CollectionAccessLevel] = UNSET,
    de: Union[Unset, List[str]] = UNSET,
    sort: Union[Unset, CollectionsGetCollectionsSort] = UNSET,
    sort_dir: Union[Unset, CollectionsGetCollectionsSortDir] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["id"] = id

    params["id_prefix"] = id_prefix

    params["name"] = name

    params["account_id"] = account_id

    params["not_account_id"] = not_account_id

    params["meta"] = meta

    params["bmeta"] = bmeta

    json_version: Union[Unset, str] = UNSET
    if not isinstance(version, Unset):
        json_version = version.value

    params["version"] = json_version

    params["version_suffix"] = version_suffix

    params["public"] = public

    params["shared_with"] = shared_with

    params["editable"] = editable

    params["starred"] = starred

    json_access_level: Union[Unset, str] = UNSET
    if not isinstance(access_level, Unset):
        json_access_level = access_level.value

    params["access_level"] = json_access_level

    json_de: Union[Unset, List[str]] = UNSET
    if not isinstance(de, Unset):
        json_de = de

    params["de"] = json_de

    json_sort: Union[Unset, str] = UNSET
    if not isinstance(sort, Unset):
        json_sort = sort.value

    params["sort"] = json_sort

    json_sort_dir: Union[Unset, str] = UNSET
    if not isinstance(sort_dir, Unset):
        json_sort_dir = sort_dir.value

    params["sort_dir"] = json_sort_dir

    params["page_size"] = page_size

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/collections",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[CollectionsGetCollectionsCollectionList, ServerError]:
    if response.status_code == HTTPStatus.OK:
        response_200 = CollectionsGetCollectionsCollectionList.from_dict(
            response.json()
        )

        return response_200
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[CollectionsGetCollectionsCollectionList, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    id: Union[Unset, str] = UNSET,
    id_prefix: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    account_id: Union[Unset, str] = UNSET,
    not_account_id: Union[Unset, str] = UNSET,
    meta: Union[Unset, str] = UNSET,
    bmeta: Union[Unset, str] = UNSET,
    version: Union[Unset, CollectionsGetCollectionsVersion] = UNSET,
    version_suffix: Union[Unset, str] = UNSET,
    public: Union[Unset, bool] = UNSET,
    shared_with: Union[Unset, str] = UNSET,
    editable: Union[Unset, bool] = UNSET,
    starred: Union[Unset, bool] = UNSET,
    access_level: Union[Unset, CollectionAccessLevel] = UNSET,
    de: Union[Unset, List[str]] = UNSET,
    sort: Union[Unset, CollectionsGetCollectionsSort] = UNSET,
    sort_dir: Union[Unset, CollectionsGetCollectionsSortDir] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Response[Union[CollectionsGetCollectionsCollectionList, ServerError]]:
    """List collections

    List all collections that are visible to the current user,
    whether logged in or not.

    Args:
        id (Union[Unset, str]):
        id_prefix (Union[Unset, str]):
        name (Union[Unset, str]):
        account_id (Union[Unset, str]):
        not_account_id (Union[Unset, str]):
        meta (Union[Unset, str]):
        bmeta (Union[Unset, str]):
        version (Union[Unset, CollectionsGetCollectionsVersion]):
        version_suffix (Union[Unset, str]):
        public (Union[Unset, bool]):
        shared_with (Union[Unset, str]):
        editable (Union[Unset, bool]):
        starred (Union[Unset, bool]):
        access_level (Union[Unset, CollectionAccessLevel]):
        de (Union[Unset, List[str]]):
        sort (Union[Unset, CollectionsGetCollectionsSort]):
        sort_dir (Union[Unset, CollectionsGetCollectionsSortDir]):
        page_size (Union[Unset, int]):  Default: 50.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CollectionsGetCollectionsCollectionList, ServerError]]
    """

    kwargs = _get_kwargs(
        id=id,
        id_prefix=id_prefix,
        name=name,
        account_id=account_id,
        not_account_id=not_account_id,
        meta=meta,
        bmeta=bmeta,
        version=version,
        version_suffix=version_suffix,
        public=public,
        shared_with=shared_with,
        editable=editable,
        starred=starred,
        access_level=access_level,
        de=de,
        sort=sort,
        sort_dir=sort_dir,
        page_size=page_size,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    id: Union[Unset, str] = UNSET,
    id_prefix: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    account_id: Union[Unset, str] = UNSET,
    not_account_id: Union[Unset, str] = UNSET,
    meta: Union[Unset, str] = UNSET,
    bmeta: Union[Unset, str] = UNSET,
    version: Union[Unset, CollectionsGetCollectionsVersion] = UNSET,
    version_suffix: Union[Unset, str] = UNSET,
    public: Union[Unset, bool] = UNSET,
    shared_with: Union[Unset, str] = UNSET,
    editable: Union[Unset, bool] = UNSET,
    starred: Union[Unset, bool] = UNSET,
    access_level: Union[Unset, CollectionAccessLevel] = UNSET,
    de: Union[Unset, List[str]] = UNSET,
    sort: Union[Unset, CollectionsGetCollectionsSort] = UNSET,
    sort_dir: Union[Unset, CollectionsGetCollectionsSortDir] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Union[CollectionsGetCollectionsCollectionList]:
    """List collections

    List all collections that are visible to the current user,
    whether logged in or not.

    Args:
        id (Union[Unset, str]):
        id_prefix (Union[Unset, str]):
        name (Union[Unset, str]):
        account_id (Union[Unset, str]):
        not_account_id (Union[Unset, str]):
        meta (Union[Unset, str]):
        bmeta (Union[Unset, str]):
        version (Union[Unset, CollectionsGetCollectionsVersion]):
        version_suffix (Union[Unset, str]):
        public (Union[Unset, bool]):
        shared_with (Union[Unset, str]):
        editable (Union[Unset, bool]):
        starred (Union[Unset, bool]):
        access_level (Union[Unset, CollectionAccessLevel]):
        de (Union[Unset, List[str]]):
        sort (Union[Unset, CollectionsGetCollectionsSort]):
        sort_dir (Union[Unset, CollectionsGetCollectionsSortDir]):
        page_size (Union[Unset, int]):  Default: 50.

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CollectionsGetCollectionsCollectionList]
    """

    response = sync_detailed(
        client=client,
        id=id,
        id_prefix=id_prefix,
        name=name,
        account_id=account_id,
        not_account_id=not_account_id,
        meta=meta,
        bmeta=bmeta,
        version=version,
        version_suffix=version_suffix,
        public=public,
        shared_with=shared_with,
        editable=editable,
        starred=starred,
        access_level=access_level,
        de=de,
        sort=sort,
        sort_dir=sort_dir,
        page_size=page_size,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    id: Union[Unset, str] = UNSET,
    id_prefix: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    account_id: Union[Unset, str] = UNSET,
    not_account_id: Union[Unset, str] = UNSET,
    meta: Union[Unset, str] = UNSET,
    bmeta: Union[Unset, str] = UNSET,
    version: Union[Unset, CollectionsGetCollectionsVersion] = UNSET,
    version_suffix: Union[Unset, str] = UNSET,
    public: Union[Unset, bool] = UNSET,
    shared_with: Union[Unset, str] = UNSET,
    editable: Union[Unset, bool] = UNSET,
    starred: Union[Unset, bool] = UNSET,
    access_level: Union[Unset, CollectionAccessLevel] = UNSET,
    de: Union[Unset, List[str]] = UNSET,
    sort: Union[Unset, CollectionsGetCollectionsSort] = UNSET,
    sort_dir: Union[Unset, CollectionsGetCollectionsSortDir] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Response[Union[CollectionsGetCollectionsCollectionList, ServerError]]:
    """List collections

    List all collections that are visible to the current user,
    whether logged in or not.

    Args:
        id (Union[Unset, str]):
        id_prefix (Union[Unset, str]):
        name (Union[Unset, str]):
        account_id (Union[Unset, str]):
        not_account_id (Union[Unset, str]):
        meta (Union[Unset, str]):
        bmeta (Union[Unset, str]):
        version (Union[Unset, CollectionsGetCollectionsVersion]):
        version_suffix (Union[Unset, str]):
        public (Union[Unset, bool]):
        shared_with (Union[Unset, str]):
        editable (Union[Unset, bool]):
        starred (Union[Unset, bool]):
        access_level (Union[Unset, CollectionAccessLevel]):
        de (Union[Unset, List[str]]):
        sort (Union[Unset, CollectionsGetCollectionsSort]):
        sort_dir (Union[Unset, CollectionsGetCollectionsSortDir]):
        page_size (Union[Unset, int]):  Default: 50.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CollectionsGetCollectionsCollectionList, ServerError]]
    """

    kwargs = _get_kwargs(
        id=id,
        id_prefix=id_prefix,
        name=name,
        account_id=account_id,
        not_account_id=not_account_id,
        meta=meta,
        bmeta=bmeta,
        version=version,
        version_suffix=version_suffix,
        public=public,
        shared_with=shared_with,
        editable=editable,
        starred=starred,
        access_level=access_level,
        de=de,
        sort=sort,
        sort_dir=sort_dir,
        page_size=page_size,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    id: Union[Unset, str] = UNSET,
    id_prefix: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    account_id: Union[Unset, str] = UNSET,
    not_account_id: Union[Unset, str] = UNSET,
    meta: Union[Unset, str] = UNSET,
    bmeta: Union[Unset, str] = UNSET,
    version: Union[Unset, CollectionsGetCollectionsVersion] = UNSET,
    version_suffix: Union[Unset, str] = UNSET,
    public: Union[Unset, bool] = UNSET,
    shared_with: Union[Unset, str] = UNSET,
    editable: Union[Unset, bool] = UNSET,
    starred: Union[Unset, bool] = UNSET,
    access_level: Union[Unset, CollectionAccessLevel] = UNSET,
    de: Union[Unset, List[str]] = UNSET,
    sort: Union[Unset, CollectionsGetCollectionsSort] = UNSET,
    sort_dir: Union[Unset, CollectionsGetCollectionsSortDir] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Union[CollectionsGetCollectionsCollectionList]:
    """List collections

    List all collections that are visible to the current user,
    whether logged in or not.

    Args:
        id (Union[Unset, str]):
        id_prefix (Union[Unset, str]):
        name (Union[Unset, str]):
        account_id (Union[Unset, str]):
        not_account_id (Union[Unset, str]):
        meta (Union[Unset, str]):
        bmeta (Union[Unset, str]):
        version (Union[Unset, CollectionsGetCollectionsVersion]):
        version_suffix (Union[Unset, str]):
        public (Union[Unset, bool]):
        shared_with (Union[Unset, str]):
        editable (Union[Unset, bool]):
        starred (Union[Unset, bool]):
        access_level (Union[Unset, CollectionAccessLevel]):
        de (Union[Unset, List[str]]):
        sort (Union[Unset, CollectionsGetCollectionsSort]):
        sort_dir (Union[Unset, CollectionsGetCollectionsSortDir]):
        page_size (Union[Unset, int]):  Default: 50.

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CollectionsGetCollectionsCollectionList]
    """

    response = await asyncio_detailed(
        client=client,
        id=id,
        id_prefix=id_prefix,
        name=name,
        account_id=account_id,
        not_account_id=not_account_id,
        meta=meta,
        bmeta=bmeta,
        version=version,
        version_suffix=version_suffix,
        public=public,
        shared_with=shared_with,
        editable=editable,
        starred=starred,
        access_level=access_level,
        de=de,
        sort=sort,
        sort_dir=sort_dir,
        page_size=page_size,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.account_update_account_body_metadata import (
    AccountUpdateAccountBodyMetadata,
)
from ..models.visibility_level import VisibilityLevel
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="AccountUpdateAccountBody")


@_attrs_define
class AccountUpdateAccountBody:
    """AccountUpdateAccountBody model

    Attributes:
        metadata (Union[Unset, AccountUpdateAccountBodyMetadata]):
        name (Union[Unset, str]):
        visibility_level (Union[Unset, VisibilityLevel]):
    """

    metadata: Union[Unset, "AccountUpdateAccountBodyMetadata"] = UNSET
    name: Union[Unset, str] = UNSET
    visibility_level: Union[Unset, VisibilityLevel] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()
        name = self.name
        visibility_level: Union[Unset, str] = UNSET
        if not isinstance(self.visibility_level, Unset):
            visibility_level = self.visibility_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if name is not UNSET:
            field_dict["name"] = name
        if visibility_level is not UNSET:
            field_dict["visibilityLevel"] = visibility_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AccountUpdateAccountBody` from a dict"""
        d = src_dict.copy()
        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, AccountUpdateAccountBodyMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = AccountUpdateAccountBodyMetadata.from_dict(_metadata)

        name = d.pop("name", UNSET)

        _visibility_level = d.pop("visibilityLevel", UNSET)
        visibility_level: Union[Unset, VisibilityLevel]
        if isinstance(_visibility_level, Unset):
            visibility_level = UNSET
        else:
            visibility_level = VisibilityLevel(_visibility_level)

        account_update_account_body = cls(
            metadata=metadata,
            name=name,
            visibility_level=visibility_level,
        )

        return account_update_account_body



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="MetadataTags")


@_attrs_define
class MetadataTags:
    """MetadataTags model

    Attributes:
        tags (List[str]):
    """

    tags: List[str]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "tags": tags,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetadataTags` from a dict"""
        d = src_dict.copy()
        tags = cast(List[str], d.pop("tags"))

        metadata_tags = cls(
            tags=tags,
        )

        return metadata_tags



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.cache_file import CacheFile
from ..models.cache_mem import CacheMem
from ..models.system_configuration_datastores_plugin_options import (
    SystemConfigurationDatastoresPluginOptions,
)
from ..models.system_configuration_datastores_plugin_type import (
    SystemConfigurationDatastoresPluginType,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SystemConfigurationDatastoresPlugin")


@_attrs_define
class SystemConfigurationDatastoresPlugin:
    """SystemConfigurationDatastoresPlugin model

    Attributes:
        plugin_name (str):
        cache (Union['CacheFile', 'CacheMem', Unset]):
        enabled (Union[Unset, bool]):  Default: True.
        is_default (Union[Unset, bool]):  Default: False.
        options (Union[Unset, SystemConfigurationDatastoresPluginOptions]):
        type (Union[Unset, SystemConfigurationDatastoresPluginType]):
    """

    plugin_name: str
    cache: Union["CacheFile", "CacheMem", Unset] = UNSET
    enabled: Union[Unset, bool] = True
    is_default: Union[Unset, bool] = False
    options: Union[Unset, "SystemConfigurationDatastoresPluginOptions"] = UNSET
    type: Union[Unset, SystemConfigurationDatastoresPluginType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        plugin_name = self.plugin_name
        cache: Union[Dict[str, Any], Unset]
        if isinstance(self.cache, Unset):
            cache = UNSET
        elif isinstance(self.cache, CacheMem):
            cache = self.cache.to_dict()
        else:
            cache = self.cache.to_dict()

        enabled = self.enabled
        is_default = self.is_default
        options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.options, Unset):
            options = self.options.to_dict()
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "plugin_name": plugin_name,
            }
        )
        if cache is not UNSET:
            field_dict["cache"] = cache
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if is_default is not UNSET:
            field_dict["is_default"] = is_default
        if options is not UNSET:
            field_dict["options"] = options
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SystemConfigurationDatastoresPlugin` from a dict"""
        d = src_dict.copy()
        plugin_name = d.pop("plugin_name")

        def _parse_cache(data: object) -> Union["CacheFile", "CacheMem", Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasconfig_datastore_cache_type_0 = CacheMem.from_dict(
                    data
                )

                return componentsschemasconfig_datastore_cache_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemasconfig_datastore_cache_type_1 = CacheFile.from_dict(data)

            return componentsschemasconfig_datastore_cache_type_1

        cache = _parse_cache(d.pop("cache", UNSET))

        enabled = d.pop("enabled", UNSET)

        is_default = d.pop("is_default", UNSET)

        _options = d.pop("options", UNSET)
        options: Union[Unset, SystemConfigurationDatastoresPluginOptions]
        if isinstance(_options, Unset):
            options = UNSET
        else:
            options = SystemConfigurationDatastoresPluginOptions.from_dict(_options)

        _type = d.pop("type", UNSET)
        type: Union[Unset, SystemConfigurationDatastoresPluginType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = SystemConfigurationDatastoresPluginType(_type)

        system_configuration_datastores_plugin = cls(
            plugin_name=plugin_name,
            cache=cache,
            enabled=enabled,
            is_default=is_default,
            options=options,
            type=type,
        )

        system_configuration_datastores_plugin.additional_properties = d
        return system_configuration_datastores_plugin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.error import Error
from ..models.pagination import Pagination
from ..models.table_data_data_item import TableDataDataItem
from ..models.table_data_data_model import TableDataDataModel


T = TypeVar("T", bound="TableData")


@_attrs_define
class TableData:
    """Dataview contents

    Attributes:
        data (List['TableDataDataItem']):
        data_model (TableDataDataModel):
        errors (List['Error']):
        pagination (Pagination):
    """

    data: List["TableDataDataItem"]
    data_model: "TableDataDataModel"
    errors: List["Error"]
    pagination: "Pagination"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)

        data_model = self.data_model.to_dict()
        errors = []
        for errors_item_data in self.errors:
            errors_item = errors_item_data.to_dict()
            errors.append(errors_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "data": data,
                "data_model": data_model,
                "errors": errors,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`TableData` from a dict"""
        d = src_dict.copy()
        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = TableDataDataItem.from_dict(data_item_data)

            data.append(data_item)

        data_model = TableDataDataModel.from_dict(d.pop("data_model"))

        errors = []
        _errors = d.pop("errors")
        for errors_item_data in _errors:
            errors_item = Error.from_dict(errors_item_data)

            errors.append(errors_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        table_data = cls(
            data=data,
            data_model=data_model,
            errors=errors,
            pagination=pagination,
        )

        return table_data



from pydelfini.delfini_core.api.group import group_create_group
from pydelfini.delfini_core.api.group import group_delete_group
from pydelfini.delfini_core.api.group import group_get_group_members
from pydelfini.delfini_core.api.group import group_get_groups
from pydelfini.delfini_core.api.group import group_set_group_member
from pydelfini.delfini_core.api.group import group_update_group
from pydelfini.delfini_core.models import GroupCreateGroupBody
from pydelfini.delfini_core.models import GroupMember
from pydelfini.delfini_core.models import GroupRole
from pydelfini.delfini_core.models import GroupUpdateGroupBody
from pydelfini.delfini_core.models import VisibilityLevel

from .base_commands import BaseCommands


class GroupsCommands(BaseCommands):
    """Operations on groups"""

    @BaseCommands._with_arg("name")
    @BaseCommands._with_arg("--public", action="store_true")
    def new(self) -> None:
        """Create a new group."""
        rv = group_create_group.sync(
            body=GroupCreateGroupBody(
                name=self.args.name,
                visibility_level=(
                    VisibilityLevel.PUBLIC
                    if self.args.public
                    else VisibilityLevel.UNLISTED
                ),
            ),
            client=self.core,
        )

        self._output(rv.to_dict())

    def list(self) -> None:
        """List all visible groups."""
        rv = group_get_groups.sync(client=self.core)

        self._output(rv.to_dict())

    @BaseCommands._with_arg("--name")
    @BaseCommands._with_arg("--make-public", action="store_true")
    @BaseCommands._with_arg("--make-unlisted", action="store_true")
    @BaseCommands._with_arg("group_id")
    def update(self) -> None:
        """Update a group."""
        update = GroupUpdateGroupBody()
        if self.args.name:
            update.name = self.args.name
        if self.args.make_public:
            update.visibility_level = VisibilityLevel.PUBLIC
        if self.args.make_unlisted:
            update.visibility_level = VisibilityLevel.UNLISTED
        group_update_group.sync(
            group_id=self.args.group_id, body=update, client=self.core
        )

    @BaseCommands._with_arg("group_id")
    def list_members(self) -> None:
        """List members of a group."""
        rv = group_get_group_members.sync(
            group_id=self.args.group_id, with_users=False, client=self.core
        )
        self._output(rv.to_dict())

    @BaseCommands._with_arg("group_id")
    @BaseCommands._with_arg("user_id")
    @BaseCommands._with_arg(
        "--role", choices=["ADMIN", "MEMBER", "VIEWER"], default="MEMBER"
    )
    def join(self) -> None:
        """Join a user to a group."""
        role = GroupRole(self.args.role)
        group_set_group_member.sync(
            group_id=self.args.group_id,
            body=GroupMember(role=role, user_id=self.args.user_id),
            client=self.core,
        )

    @BaseCommands._with_arg("group_id")
    def delete(self) -> None:
        """Remove a group."""
        rv = group_delete_group.sync(group_id=self.args.group_id, client=self.core)
        assert rv



import json
import sys
from typing import Any

import yaml
from pydelfini.delfini_core.api.admin import admin_get_config
from pydelfini.delfini_core.api.admin import admin_put_config
from pydelfini.delfini_core.models import SystemConfiguration

from .base_commands import BaseCommands


def setattr_conv(d: dict[str, Any], key: str, value: str) -> None:
    """Set a dictionary key, converting value if needed."""
    new_value: Any
    if key in d:
        old_value = d[key]

        if isinstance(old_value, str):
            new_value = value
        elif isinstance(old_value, bool):
            new_value = bool(json.loads(value.lower()))
        elif isinstance(old_value, int):
            new_value = int(value)
        elif isinstance(old_value, float):
            new_value = float(value)
        else:
            new_value = json.loads(value)
    else:
        try:
            new_value = json.loads(value)
        except json.JSONDecodeError:
            new_value = value

    d[key] = new_value


def traverse_path(path: str, root: dict[str, Any]) -> tuple[str, dict[str, Any]]:
    path_list = list(reversed(path.split(".")))
    obj = root
    while len(path_list) > 1:
        next_path = path_list.pop()
        obj = obj[next_path]

    assert isinstance(obj, dict)

    return path_list.pop(), obj


class ConfigCommands(BaseCommands):
    """Admin operations on Delfini configuration"""

    def get(self) -> None:
        """Retrieve the current configuration."""
        config = admin_get_config.sync(client=self.core)

        self._output(config.to_dict())

    @BaseCommands._with_arg("config_path", help="dot-separated path to config option")
    @BaseCommands._with_arg("value", help="new value to set")
    def set(self) -> None:
        """Set a single config value."""
        config = admin_get_config.sync(client=self.core).to_dict()

        name, obj = traverse_path(self.args.config_path, config)
        setattr_conv(obj, name, self.args.value)

        self._output(config, "--- setting new config ---")
        admin_put_config.sync(
            body=SystemConfiguration.from_dict(config), client=self.core
        )

    @BaseCommands._with_arg("config_path", help="dot-separated path to config option")
    def delete(self) -> None:
        """Remove a node from the config structure."""
        config = admin_get_config.sync(client=self.core).to_dict()

        name, obj = traverse_path(self.args.config_path, config)
        del obj[name]

        self._output(config, "--- setting new config ---")
        admin_put_config.sync(
            body=SystemConfiguration.from_dict(config), client=self.core
        )

    @BaseCommands._with_arg(
        "filename", default="-", help="complete configuration in YAML format"
    )
    def put(self) -> None:
        """Write the entire configuration."""
        if self.args.filename == "-":
            input_stream = sys.stdin
        else:
            input_stream = open(self.args.filename)

        config = yaml.safe_load(input_stream)

        admin_put_config.sync(
            body=SystemConfiguration.from_dict(config), client=self.core
        )



from enum import Enum


class GroupRole(str, Enum):
    ADMIN = "ADMIN"
    AUTHOR = "AUTHOR"
    MEMBER = "MEMBER"
    VIEWER = "VIEWER"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.group_role import GroupRole
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="GroupMember")


@_attrs_define
class GroupMember:
    """GroupMember model

    Attributes:
        role (GroupRole):
        user_id (str):
        group_id (Union[Unset, str]):
    """

    role: GroupRole
    user_id: str
    group_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        role = self.role.value
        user_id = self.user_id
        group_id = self.group_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "role": role,
                "user_id": user_id,
            }
        )
        if group_id is not UNSET:
            field_dict["group_id"] = group_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`GroupMember` from a dict"""
        d = src_dict.copy()
        role = GroupRole(d.pop("role"))

        user_id = d.pop("user_id")

        group_id = d.pop("group_id", UNSET)

        group_member = cls(
            role=role,
            user_id=user_id,
            group_id=group_id,
        )

        return group_member



import datetime
from typing import Any
from typing import cast
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="UserAdmin")


@_attrs_define
class UserAdmin:
    """User administrative settings

    Attributes:
        disabled_on (Union[None, Unset, datetime.datetime]):
    """

    disabled_on: Union[None, Unset, datetime.datetime] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        disabled_on: Union[None, Unset, str]
        if isinstance(self.disabled_on, Unset):
            disabled_on = UNSET
        elif isinstance(self.disabled_on, datetime.datetime):
            disabled_on = self.disabled_on.isoformat()
        else:
            disabled_on = self.disabled_on

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if disabled_on is not UNSET:
            field_dict["disabled_on"] = disabled_on

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`UserAdmin` from a dict"""
        d = src_dict.copy()

        def _parse_disabled_on(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                disabled_on_type_0 = isoparse(data)

                return disabled_on_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        disabled_on = _parse_disabled_on(d.pop("disabled_on", UNSET))

        user_admin = cls(
            disabled_on=disabled_on,
        )

        return user_admin



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.datastore import Datastore


T = TypeVar("T", bound="RootGetDatastoresResponse200")


@_attrs_define
class RootGetDatastoresResponse200:
    """RootGetDatastoresResponse200 model

    Attributes:
        datastores (List['Datastore']):
    """

    datastores: List["Datastore"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        datastores = []
        for datastores_item_data in self.datastores:
            datastores_item = datastores_item_data.to_dict()
            datastores.append(datastores_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "datastores": datastores,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`RootGetDatastoresResponse200` from a dict"""
        d = src_dict.copy()
        datastores = []
        _datastores = d.pop("datastores")
        for datastores_item_data in _datastores:
            datastores_item = Datastore.from_dict(datastores_item_data)

            datastores.append(datastores_item)

        root_get_datastores_response_200 = cls(
            datastores=datastores,
        )

        return root_get_datastores_response_200



from enum import Enum


class SearchSearchCollectionsBodyTypesItem(str, Enum):
    COLLECTION = "collection"
    ITEM = "item"
    ITEM_DATA_ELEMENT = "item-data-element"

    def __str__(self) -> str:
        return str(self.value)



import pytest
from pydelfini.delfini_core import AuthenticatedClient


@pytest.fixture
def client():
    return AuthenticatedClient(
        base_url="https://delfini.test/api/v1", token="test-token"
    )



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.update_user_metadata import UpdateUserMetadata
from ..models.visibility_level import VisibilityLevel
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="UpdateUser")


@_attrs_define
class UpdateUser:
    """Update properties of the user

    Attributes:
        given_name (Union[Unset, str]):
        last_name (Union[Unset, str]):
        metadata (Union[Unset, UpdateUserMetadata]):
        password (Union[Unset, str]):
        user_email (Union[Unset, str]):
        user_name (Union[Unset, str]):
        visibility_level (Union[Unset, VisibilityLevel]):
    """

    given_name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    metadata: Union[Unset, "UpdateUserMetadata"] = UNSET
    password: Union[Unset, str] = UNSET
    user_email: Union[Unset, str] = UNSET
    user_name: Union[Unset, str] = UNSET
    visibility_level: Union[Unset, VisibilityLevel] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        given_name = self.given_name
        last_name = self.last_name
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()
        password = self.password
        user_email = self.user_email
        user_name = self.user_name
        visibility_level: Union[Unset, str] = UNSET
        if not isinstance(self.visibility_level, Unset):
            visibility_level = self.visibility_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if given_name is not UNSET:
            field_dict["given_name"] = given_name
        if last_name is not UNSET:
            field_dict["last_name"] = last_name
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if password is not UNSET:
            field_dict["password"] = password
        if user_email is not UNSET:
            field_dict["user_email"] = user_email
        if user_name is not UNSET:
            field_dict["user_name"] = user_name
        if visibility_level is not UNSET:
            field_dict["visibility_level"] = visibility_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`UpdateUser` from a dict"""
        d = src_dict.copy()
        given_name = d.pop("given_name", UNSET)

        last_name = d.pop("last_name", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, UpdateUserMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = UpdateUserMetadata.from_dict(_metadata)

        password = d.pop("password", UNSET)

        user_email = d.pop("user_email", UNSET)

        user_name = d.pop("user_name", UNSET)

        _visibility_level = d.pop("visibility_level", UNSET)
        visibility_level: Union[Unset, VisibilityLevel]
        if isinstance(_visibility_level, Unset):
            visibility_level = UNSET
        else:
            visibility_level = VisibilityLevel(_visibility_level)

        update_user = cls(
            given_name=given_name,
            last_name=last_name,
            metadata=metadata,
            password=password,
            user_email=user_email,
            user_name=user_name,
            visibility_level=visibility_level,
        )

        return update_user



from enum import Enum


class DataElementConceptAppliesTo(str, Enum):
    CONCEPTUALDOMAIN = "conceptualDomain"
    DATAELEMENT = "dataElement"
    OBJECTCLASS = "objectClass"
    VALUEDOMAIN = "valueDomain"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.search_search_accounts_body_types_item import (
    SearchSearchAccountsBodyTypesItem,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SearchSearchAccountsBody")


@_attrs_define
class SearchSearchAccountsBody:
    """SearchSearchAccountsBody model

    Attributes:
        query (str):
        account_bmeta (Union[Unset, str]): Filter by account metadata.

            Provide a base64-encoded JSON mapping conformant to the
            following JSON schema:

              schema:
                type: object
                additionalProperties:
                  anyOf:
                    - type: string
                    - type: array
                      items:
                        type: string

            This includes any combination of `{"x-meta": "foo"}` and
            `{"x-meta-2": ["foo", "bar"]}`:

            * In the single string case, the provided string must
              match exactly to the value in the metadata field.

            * In the case of an array of strings, the metadata field
              is assumed to contain a JSON-encoded array of strings,
              and only one of the strings in the provided array needs
              to match one of the strings in the metadata field.
        types (Union[Unset, List[SearchSearchAccountsBodyTypesItem]]):
    """

    query: str
    account_bmeta: Union[Unset, str] = UNSET
    types: Union[Unset, List[SearchSearchAccountsBodyTypesItem]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        query = self.query
        account_bmeta = self.account_bmeta
        types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.types, Unset):
            types = []
            for types_item_data in self.types:
                types_item = types_item_data.value
                types.append(types_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "query": query,
            }
        )
        if account_bmeta is not UNSET:
            field_dict["accountBmeta"] = account_bmeta
        if types is not UNSET:
            field_dict["types"] = types

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchSearchAccountsBody` from a dict"""
        d = src_dict.copy()
        query = d.pop("query")

        account_bmeta = d.pop("accountBmeta", UNSET)

        types = []
        _types = d.pop("types", UNSET)
        for types_item_data in _types or []:
            types_item = SearchSearchAccountsBodyTypesItem(types_item_data)

            types.append(types_item)

        search_search_accounts_body = cls(
            query=query,
            account_bmeta=account_bmeta,
            types=types,
        )

        return search_search_accounts_body



"""Retrieve the status of an item"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.item_status_details import ItemStatusDetails
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    collection_id: str,
    version_id: str,
    item_id: str,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/collections/{collection_id}/{version_id}/items/{item_id}/status".format(
            collection_id=collection_id,
            version_id=version_id,
            item_id=item_id,
        ),
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[ItemStatusDetails, ServerError]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ItemStatusDetails.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ServerError.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[ItemStatusDetails, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[ItemStatusDetails, ServerError]]:
    """Retrieve the status of an item

    Get a detailed view of the status of an item, along with any errors.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ItemStatusDetails, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
) -> Union[ItemStatusDetails]:
    """Retrieve the status of an item

    Get a detailed view of the status of an item, along with any errors.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ItemStatusDetails]
    """

    response = sync_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[ItemStatusDetails, ServerError]]:
    """Retrieve the status of an item

    Get a detailed view of the status of an item, along with any errors.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ItemStatusDetails, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
) -> Union[ItemStatusDetails]:
    """Retrieve the status of an item

    Get a detailed view of the status of an item, along with any errors.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ItemStatusDetails]
    """

    response = await asyncio_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from enum import Enum


class CollectionRole(str, Enum):
    ADMIN = "ADMIN"
    READER = "READER"
    VIEWER = "VIEWER"
    WRITER = "WRITER"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class DataElementDefinitionDefinitionType(str, Enum):
    ALSOKNOWNAS = "alsoKnownAs"
    ALTERNATEQUESTIONTEXT = "alternateQuestionText"
    ASSOCIATEDWITH = "associatedWith"
    COLUMNNAME = "columnName"
    LONGDESCRIPTION = "longDescription"
    OTHER = "other"
    PREFERREDQUESTIONTEXT = "preferredQuestionText"
    SHORTDESCRIPTION = "shortDescription"
    SOURCE = "source"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.dictionary_search_options import DictionarySearchOptions


T = TypeVar("T", bound="SearchGetDictionarySearchOptionsResponse200")


@_attrs_define
class SearchGetDictionarySearchOptionsResponse200:
    """SearchGetDictionarySearchOptionsResponse200 model

    Attributes:
        sources (List['DictionarySearchOptions']):
    """

    sources: List["DictionarySearchOptions"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        sources = []
        for sources_item_data in self.sources:
            sources_item = sources_item_data.to_dict()
            sources.append(sources_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "sources": sources,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchGetDictionarySearchOptionsResponse200` from a dict"""
        d = src_dict.copy()
        sources = []
        _sources = d.pop("sources")
        for sources_item_data in _sources:
            sources_item = DictionarySearchOptions.from_dict(sources_item_data)

            sources.append(sources_item)

        search_get_dictionary_search_options_response_200 = cls(
            sources=sources,
        )

        return search_get_dictionary_search_options_response_200



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar(
    "T", bound="DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1"
)


@_attrs_define
class DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1:
    """DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1 model

    Attributes:
        strptime (str): Datetime parsing format string accepting
            percent-prefix format specifiers as in the
            standard strptime library function
    """

    strptime: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        strptime = self.strptime

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "strptime": strptime,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesDateTimeFormatDateTimeFormatFormatType1` from a dict"""
        d = src_dict.copy()
        strptime = d.pop("strptime")

        data_element_permissible_values_date_time_format_date_time_format_format_type_1 = cls(
            strptime=strptime,
        )

        return data_element_permissible_values_date_time_format_date_time_format_format_type_1



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field


T = TypeVar("T", bound="MetricTimepointPartitions")


@_attrs_define
class MetricTimepointPartitions:
    """MetricTimepointPartitions model"""

    additional_properties: Dict[str, Union[float, int]] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetricTimepointPartitions` from a dict"""
        d = src_dict.copy()
        metric_timepoint_partitions = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():

            def _parse_additional_property(data: object) -> Union[float, int]:
                return cast(Union[float, int], data)

            additional_property = _parse_additional_property(prop_dict)

            additional_properties[prop_name] = additional_property

        metric_timepoint_partitions.additional_properties = additional_properties
        return metric_timepoint_partitions

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Union[float, int]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Union[float, int]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.account import Account
from ..models.highlights import Highlights
from ..models.search_accounts_response_hits_item_projects_item import (
    SearchAccountsResponseHitsItemProjectsItem,
)


T = TypeVar("T", bound="SearchAccountsResponseHitsItem")


@_attrs_define
class SearchAccountsResponseHitsItem:
    """SearchAccountsResponseHitsItem model

    Attributes:
        account (Account):
        highlights (Highlights):
        projects (List['SearchAccountsResponseHitsItemProjectsItem']):
        score (float):
    """

    account: "Account"
    highlights: "Highlights"
    projects: List["SearchAccountsResponseHitsItemProjectsItem"]
    score: float

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        account = self.account.to_dict()
        highlights = self.highlights.to_dict()
        projects = []
        for projects_item_data in self.projects:
            projects_item = projects_item_data.to_dict()
            projects.append(projects_item)

        score = self.score

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "account": account,
                "highlights": highlights,
                "projects": projects,
                "score": score,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchAccountsResponseHitsItem` from a dict"""
        d = src_dict.copy()
        account = Account.from_dict(d.pop("account"))

        highlights = Highlights.from_dict(d.pop("highlights"))

        projects = []
        _projects = d.pop("projects")
        for projects_item_data in _projects:
            projects_item = SearchAccountsResponseHitsItemProjectsItem.from_dict(
                projects_item_data
            )

            projects.append(projects_item)

        score = d.pop("score")

        search_accounts_response_hits_item = cls(
            account=account,
            highlights=highlights,
            projects=projects,
            score=score,
        )

        return search_accounts_response_hits_item



import argparse
import json
import sys
import textwrap
import types
from typing import Any
from typing import Callable
from typing import Optional

import yaml
from pydelfini.client import DelfiniClient
from tabulate import tabulate


class BaseCommands:
    _func_args: dict[Callable[..., None], list[tuple[tuple[Any, ...], dict[str, Any]]]]

    def __init__(self, client: DelfiniClient, args: argparse.Namespace) -> None:
        self.client = client
        self.core = client._client
        self.args = args

    def _output(
        self,
        data: Any,
        header: Optional[str] = None,
        preferred_type: Optional[str] = None,
    ) -> None:
        output_type = self.args.output_type
        if output_type == "auto":
            output_type = preferred_type or "yaml"

        if output_type == "yaml":
            if header:
                print("#", header)
            print(yaml.dump(data))

        elif output_type == "json":
            print(json.dumps(data, indent=2))

        elif output_type == "table":
            try:
                print(tabulate(data, headers="keys"))
            except TypeError:
                print(
                    'This command does not support "table" output. Please choose'
                    " another output format."
                )
                sys.exit(1)
        else:
            raise ValueError(f"unknown output type: {output_type}")

    @classmethod
    def _add_to_parser(
        cls, subparsers: "argparse._SubParsersAction[argparse.ArgumentParser]"
    ) -> None:
        for method in dir(cls):
            if method.startswith("_"):
                continue

            action = getattr(cls, method)
            action_doc = action.__doc__ if action.__doc__ is not None else ""
            help_line = (
                action_doc.splitlines()[0].strip().rstrip(".") if action_doc else ""
            )
            parser = subparsers.add_parser(
                method.replace("_", "-"),
                help=help_line,
                description=(
                    help_line
                    + "\n\n"
                    + textwrap.dedent("\n".join(action_doc.split("\n")[1:]))
                ),
                formatter_class=argparse.RawDescriptionHelpFormatter,
            )

            if type(action) is type(object) and issubclass(action, BaseCommands):
                action._add_to_parser(parser.add_subparsers(title="subcommands"))

            elif isinstance(action, types.FunctionType):
                if hasattr(cls, "_func_args"):
                    for args, kwargs in cls._func_args.get(action, []):
                        parser.add_argument(*args, **kwargs)
                parser.set_defaults(cls=cls, func=action)

    @classmethod
    def _with_arg(cls, *args: Any, **kwargs: Any) -> Callable[..., Callable[..., None]]:
        def _inner(func: Callable[..., None]) -> Callable[..., None]:
            if not hasattr(cls, "_func_args"):
                cls._func_args = {}
            cls._func_args.setdefault(func, []).insert(0, (args, kwargs))
            return func

        return _inner



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.account_create_account_body_metadata import (
    AccountCreateAccountBodyMetadata,
)
from ..models.visibility_level import VisibilityLevel
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="AccountCreateAccountBody")


@_attrs_define
class AccountCreateAccountBody:
    """AccountCreateAccountBody model

    Attributes:
        metadata (AccountCreateAccountBodyMetadata):
        name (str):
        visibility_level (Union[Unset, VisibilityLevel]):
    """

    metadata: "AccountCreateAccountBodyMetadata"
    name: str
    visibility_level: Union[Unset, VisibilityLevel] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        metadata = self.metadata.to_dict()
        name = self.name
        visibility_level: Union[Unset, str] = UNSET
        if not isinstance(self.visibility_level, Unset):
            visibility_level = self.visibility_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "metadata": metadata,
                "name": name,
            }
        )
        if visibility_level is not UNSET:
            field_dict["visibilityLevel"] = visibility_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AccountCreateAccountBody` from a dict"""
        d = src_dict.copy()
        metadata = AccountCreateAccountBodyMetadata.from_dict(d.pop("metadata"))

        name = d.pop("name")

        _visibility_level = d.pop("visibilityLevel", UNSET)
        visibility_level: Union[Unset, VisibilityLevel]
        if isinstance(_visibility_level, Unset):
            visibility_level = UNSET
        else:
            visibility_level = VisibilityLevel(_visibility_level)

        account_create_account_body = cls(
            metadata=metadata,
            name=name,
            visibility_level=visibility_level,
        )

        return account_create_account_body



"""Retrieve real-time aggregated metric data from an event"""
import datetime
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.metric_agg_func import MetricAggFunc
from ...models.metric_data import MetricData
from ...models.metrics_query_event_data_period import MetricsQueryEventDataPeriod
from ...models.server_error import ServerError
from ...types import Response
from ...types import UNSET
from ...types import Unset


def _get_kwargs(
    event_name: str,
    *,
    dimension: Union[Unset, str] = UNSET,
    period: MetricsQueryEventDataPeriod,
    agg_func: MetricAggFunc,
    partition_by: Union[Unset, str] = UNSET,
    num_partitions: Union[Unset, int] = UNSET,
    start: Union[Unset, datetime.datetime] = UNSET,
    end: Union[Unset, datetime.datetime] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["dimension"] = dimension

    json_period = period.value
    params["period"] = json_period

    json_agg_func = agg_func.value
    params["agg_func"] = json_agg_func

    params["partition_by"] = partition_by

    params["num_partitions"] = num_partitions

    json_start: Union[Unset, str] = UNSET
    if not isinstance(start, Unset):
        json_start = start.isoformat()
    params["start"] = json_start

    json_end: Union[Unset, str] = UNSET
    if not isinstance(end, Unset):
        json_end = end.isoformat()
    params["end"] = json_end

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/metrics/events/{event_name}/data".format(
            event_name=event_name,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[MetricData, ServerError]:
    if response.status_code == HTTPStatus.OK:
        response_200 = MetricData.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[MetricData, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    event_name: str,
    *,
    client: AuthenticatedClient,
    dimension: Union[Unset, str] = UNSET,
    period: MetricsQueryEventDataPeriod,
    agg_func: MetricAggFunc,
    partition_by: Union[Unset, str] = UNSET,
    num_partitions: Union[Unset, int] = UNSET,
    start: Union[Unset, datetime.datetime] = UNSET,
    end: Union[Unset, datetime.datetime] = UNSET,
) -> Response[Union[MetricData, ServerError]]:
    """Retrieve real-time aggregated metric data from an event

    Args:
        event_name (str):
        dimension (Union[Unset, str]):
        period (MetricsQueryEventDataPeriod):
        agg_func (MetricAggFunc):
        partition_by (Union[Unset, str]):
        num_partitions (Union[Unset, int]):
        start (Union[Unset, datetime.datetime]):
        end (Union[Unset, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[MetricData, ServerError]]
    """

    kwargs = _get_kwargs(
        event_name=event_name,
        dimension=dimension,
        period=period,
        agg_func=agg_func,
        partition_by=partition_by,
        num_partitions=num_partitions,
        start=start,
        end=end,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    event_name: str,
    *,
    client: AuthenticatedClient,
    dimension: Union[Unset, str] = UNSET,
    period: MetricsQueryEventDataPeriod,
    agg_func: MetricAggFunc,
    partition_by: Union[Unset, str] = UNSET,
    num_partitions: Union[Unset, int] = UNSET,
    start: Union[Unset, datetime.datetime] = UNSET,
    end: Union[Unset, datetime.datetime] = UNSET,
) -> Union[MetricData]:
    """Retrieve real-time aggregated metric data from an event

    Args:
        event_name (str):
        dimension (Union[Unset, str]):
        period (MetricsQueryEventDataPeriod):
        agg_func (MetricAggFunc):
        partition_by (Union[Unset, str]):
        num_partitions (Union[Unset, int]):
        start (Union[Unset, datetime.datetime]):
        end (Union[Unset, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[MetricData]
    """

    response = sync_detailed(
        event_name=event_name,
        client=client,
        dimension=dimension,
        period=period,
        agg_func=agg_func,
        partition_by=partition_by,
        num_partitions=num_partitions,
        start=start,
        end=end,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    event_name: str,
    *,
    client: AuthenticatedClient,
    dimension: Union[Unset, str] = UNSET,
    period: MetricsQueryEventDataPeriod,
    agg_func: MetricAggFunc,
    partition_by: Union[Unset, str] = UNSET,
    num_partitions: Union[Unset, int] = UNSET,
    start: Union[Unset, datetime.datetime] = UNSET,
    end: Union[Unset, datetime.datetime] = UNSET,
) -> Response[Union[MetricData, ServerError]]:
    """Retrieve real-time aggregated metric data from an event

    Args:
        event_name (str):
        dimension (Union[Unset, str]):
        period (MetricsQueryEventDataPeriod):
        agg_func (MetricAggFunc):
        partition_by (Union[Unset, str]):
        num_partitions (Union[Unset, int]):
        start (Union[Unset, datetime.datetime]):
        end (Union[Unset, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[MetricData, ServerError]]
    """

    kwargs = _get_kwargs(
        event_name=event_name,
        dimension=dimension,
        period=period,
        agg_func=agg_func,
        partition_by=partition_by,
        num_partitions=num_partitions,
        start=start,
        end=end,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    event_name: str,
    *,
    client: AuthenticatedClient,
    dimension: Union[Unset, str] = UNSET,
    period: MetricsQueryEventDataPeriod,
    agg_func: MetricAggFunc,
    partition_by: Union[Unset, str] = UNSET,
    num_partitions: Union[Unset, int] = UNSET,
    start: Union[Unset, datetime.datetime] = UNSET,
    end: Union[Unset, datetime.datetime] = UNSET,
) -> Union[MetricData]:
    """Retrieve real-time aggregated metric data from an event

    Args:
        event_name (str):
        dimension (Union[Unset, str]):
        period (MetricsQueryEventDataPeriod):
        agg_func (MetricAggFunc):
        partition_by (Union[Unset, str]):
        num_partitions (Union[Unset, int]):
        start (Union[Unset, datetime.datetime]):
        end (Union[Unset, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[MetricData]
    """

    response = await asyncio_detailed(
        event_name=event_name,
        client=client,
        dimension=dimension,
        period=period,
        agg_func=agg_func,
        partition_by=partition_by,
        num_partitions=num_partitions,
        start=start,
        end=end,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SearchDictionariesInverseResultQueriesItem")


@_attrs_define
class SearchDictionariesInverseResultQueriesItem:
    """SearchDictionariesInverseResultQueriesItem model

    Attributes:
        index (int):
        query (str):
        rank (float):
        validates (Union[Unset, bool]):
    """

    index: int
    query: str
    rank: float
    validates: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        index = self.index
        query = self.query
        rank = self.rank
        validates = self.validates

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "index": index,
                "query": query,
                "rank": rank,
            }
        )
        if validates is not UNSET:
            field_dict["validates"] = validates

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchDictionariesInverseResultQueriesItem` from a dict"""
        d = src_dict.copy()
        index = d.pop("index")

        query = d.pop("query")

        rank = d.pop("rank")

        validates = d.pop("validates", UNSET)

        search_dictionaries_inverse_result_queries_item = cls(
            index=index,
            query=query,
            rank=rank,
            validates=validates,
        )

        return search_dictionaries_inverse_result_queries_item



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="CollectionMoveDatastore")


@_attrs_define
class CollectionMoveDatastore:
    """CollectionMoveDatastore model

    Attributes:
        destination_datastore (str):
    """

    destination_datastore: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        destination_datastore = self.destination_datastore

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "destinationDatastore": destination_datastore,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionMoveDatastore` from a dict"""
        d = src_dict.copy()
        destination_datastore = d.pop("destinationDatastore")

        collection_move_datastore = cls(
            destination_datastore=destination_datastore,
        )

        return collection_move_datastore



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.metric_timepoint import MetricTimepoint


T = TypeVar("T", bound="MetricData")


@_attrs_define
class MetricData:
    """MetricData model

    Attributes:
        data (List['MetricTimepoint']):
    """

    data: List["MetricTimepoint"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "data": data,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetricData` from a dict"""
        d = src_dict.copy()
        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = MetricTimepoint.from_dict(data_item_data)

            data.append(data_item)

        metric_data = cls(
            data=data,
        )

        return metric_data



"""Create a new immutable version of an existing collection"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.collection import Collection
from ...models.new_collection_version import NewCollectionVersion
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    collection_id: str,
    *,
    body: NewCollectionVersion,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/collections/{collection_id}/LIVE".format(
            collection_id=collection_id,
        ),
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[Collection, ServerError]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = Collection.from_dict(response.json())

        return response_201
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ServerError.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE:
        response_415 = ServerError.from_dict(response.json())

        return response_415
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Collection, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionVersion,
) -> Response[Union[Collection, ServerError]]:
    """Create a new immutable version of an existing collection

    This method is used to create an immutable version of a
    collection. The resulting collection version will be a
    read-only copy of the provided collection ID.

    Optionally, in the request body, a list of `itemIds` can be
    specified which will cause the new version to only include the
    items listed, which must be drawn from the set of items
    already present in the collection.

    Args:
        collection_id (str):
        body (NewCollectionVersion): Collection version details

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Collection, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionVersion,
) -> Union[Collection]:
    """Create a new immutable version of an existing collection

    This method is used to create an immutable version of a
    collection. The resulting collection version will be a
    read-only copy of the provided collection ID.

    Optionally, in the request body, a list of `itemIds` can be
    specified which will cause the new version to only include the
    items listed, which must be drawn from the set of items
    already present in the collection.

    Args:
        collection_id (str):
        body (NewCollectionVersion): Collection version details

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Collection]
    """

    response = sync_detailed(
        collection_id=collection_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    collection_id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionVersion,
) -> Response[Union[Collection, ServerError]]:
    """Create a new immutable version of an existing collection

    This method is used to create an immutable version of a
    collection. The resulting collection version will be a
    read-only copy of the provided collection ID.

    Optionally, in the request body, a list of `itemIds` can be
    specified which will cause the new version to only include the
    items listed, which must be drawn from the set of items
    already present in the collection.

    Args:
        collection_id (str):
        body (NewCollectionVersion): Collection version details

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Collection, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    *,
    client: AuthenticatedClient,
    body: NewCollectionVersion,
) -> Union[Collection]:
    """Create a new immutable version of an existing collection

    This method is used to create an immutable version of a
    collection. The resulting collection version will be a
    read-only copy of the provided collection ID.

    Optionally, in the request body, a list of `itemIds` can be
    specified which will cause the new version to only include the
    items listed, which must be drawn from the set of items
    already present in the collection.

    Args:
        collection_id (str):
        body (NewCollectionVersion): Collection version details

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Collection]
    """

    response = await asyncio_detailed(
        collection_id=collection_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="PreferredOrder")


@_attrs_define
class PreferredOrder:
    """PreferredOrder model

    Attributes:
        order (List[str]):
    """

    order: List[str]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        order = self.order

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "order": order,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`PreferredOrder` from a dict"""
        d = src_dict.copy()
        order = cast(List[str], d.pop("order"))

        preferred_order = cls(
            order=order,
        )

        return preferred_order



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="CollectionsItemsCompleteMultipartUploadBodyPartsItem")


@_attrs_define
class CollectionsItemsCompleteMultipartUploadBodyPartsItem:
    """CollectionsItemsCompleteMultipartUploadBodyPartsItem model

    Attributes:
        e_tag (str):
        part_number (int):
    """

    e_tag: str
    part_number: int

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        e_tag = self.e_tag
        part_number = self.part_number

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "ETag": e_tag,
                "PartNumber": part_number,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsItemsCompleteMultipartUploadBodyPartsItem` from a dict"""
        d = src_dict.copy()
        e_tag = d.pop("ETag")

        part_number = d.pop("PartNumber")

        collections_items_complete_multipart_upload_body_parts_item = cls(
            e_tag=e_tag,
            part_number=part_number,
        )

        return collections_items_complete_multipart_upload_body_parts_item



import datetime
from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.task_def_data_type_4 import TaskDefDataType4
from ..models.task_def_retry_item import TaskDefRetryItem
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="TaskDef")


@_attrs_define
class TaskDef:
    """TaskDef model

    Attributes:
        action (str):
        deadline (Union[datetime.datetime, float]):
        retry (List['TaskDefRetryItem']):
        data (Union['TaskDefDataType4', List[Any], Unset, float, int, str]):
        id (Union[Unset, str]):
        idemkey (Union[Unset, str]):
        priority (Union[Unset, int]):  Default: 0.
    """

    action: str
    deadline: Union[datetime.datetime, float]
    retry: List["TaskDefRetryItem"]
    data: Union["TaskDefDataType4", List[Any], Unset, float, int, str] = UNSET
    id: Union[Unset, str] = UNSET
    idemkey: Union[Unset, str] = UNSET
    priority: Union[Unset, int] = 0
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        action = self.action
        deadline: Union[float, str]
        if isinstance(self.deadline, datetime.datetime):
            deadline = self.deadline.isoformat()
        else:
            deadline = self.deadline
        retry = []
        for retry_item_data in self.retry:
            retry_item = retry_item_data.to_dict()
            retry.append(retry_item)

        data: Union[Dict[str, Any], List[Any], Unset, float, int, str]
        if isinstance(self.data, Unset):
            data = UNSET
        elif isinstance(self.data, list):
            data = self.data

        elif isinstance(self.data, TaskDefDataType4):
            data = self.data.to_dict()
        else:
            data = self.data
        id = self.id
        idemkey = self.idemkey
        priority = self.priority

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "action": action,
                "deadline": deadline,
                "retry": retry,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data
        if id is not UNSET:
            field_dict["id"] = id
        if idemkey is not UNSET:
            field_dict["idemkey"] = idemkey
        if priority is not UNSET:
            field_dict["priority"] = priority

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`TaskDef` from a dict"""
        d = src_dict.copy()
        action = d.pop("action")

        def _parse_deadline(data: object) -> Union[datetime.datetime, float]:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                deadline_type_0 = isoparse(data)

                return deadline_type_0
            except:  # noqa: E722
                pass
            return cast(Union[datetime.datetime, float], data)

        deadline = _parse_deadline(d.pop("deadline"))

        retry = []
        _retry = d.pop("retry")
        for retry_item_data in _retry:
            retry_item = TaskDefRetryItem.from_dict(retry_item_data)

            retry.append(retry_item)

        def _parse_data(
            data: object,
        ) -> Union["TaskDefDataType4", List[Any], Unset, float, int, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                data_type_3 = cast(List[Any], data)

                return data_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                data_type_4 = TaskDefDataType4.from_dict(data)

                return data_type_4
            except:  # noqa: E722
                pass
            return cast(
                Union["TaskDefDataType4", List[Any], Unset, float, int, str], data
            )

        data = _parse_data(d.pop("data", UNSET))

        id = d.pop("id", UNSET)

        idemkey = d.pop("idemkey", UNSET)

        priority = d.pop("priority", UNSET)

        task_def = cls(
            action=action,
            deadline=deadline,
            retry=retry,
            data=data,
            id=id,
            idemkey=idemkey,
            priority=priority,
        )

        task_def.additional_properties = d
        return task_def

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.event_metric import EventMetric
from ..models.metadata_metric import MetadataMetric


T = TypeVar("T", bound="MetricList")


@_attrs_define
class MetricList:
    """MetricList model

    Attributes:
        metrics (List[Union['EventMetric', 'MetadataMetric']]):
    """

    metrics: List[Union["EventMetric", "MetadataMetric"]]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        metrics = []
        for metrics_item_data in self.metrics:
            metrics_item: Dict[str, Any]
            if isinstance(metrics_item_data, MetadataMetric):
                metrics_item = metrics_item_data.to_dict()
            else:
                metrics_item = metrics_item_data.to_dict()

            metrics.append(metrics_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "metrics": metrics,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetricList` from a dict"""
        d = src_dict.copy()
        metrics = []
        _metrics = d.pop("metrics")
        for metrics_item_data in _metrics:

            def _parse_metrics_item(
                data: object,
            ) -> Union["EventMetric", "MetadataMetric"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemasmetric_type_0 = MetadataMetric.from_dict(data)

                    return componentsschemasmetric_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasmetric_type_1 = EventMetric.from_dict(data)

                return componentsschemasmetric_type_1

            metrics_item = _parse_metrics_item(metrics_item_data)

            metrics.append(metrics_item)

        metric_list = cls(
            metrics=metrics,
        )

        return metric_list



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.item_column import ItemColumn
from ..models.item_parser import ItemParser
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionsTablesPreviewTableDataBody")


@_attrs_define
class CollectionsTablesPreviewTableDataBody:
    """CollectionsTablesPreviewTableDataBody model

    Attributes:
        parser (Union['ItemParser', None]):  Example: {'name': 'auto'}.
        columns (Union[List['ItemColumn'], None, Unset]):
        dataview_definition (Union[Unset, str]): PRQL or SQL code
        max_rows (Union[Unset, int]):  Default: 25.
    """

    parser: Union["ItemParser", None]
    columns: Union[List["ItemColumn"], None, Unset] = UNSET
    dataview_definition: Union[Unset, str] = UNSET
    max_rows: Union[Unset, int] = 25

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        parser: Union[Dict[str, Any], None]
        if isinstance(self.parser, ItemParser):
            parser = self.parser.to_dict()
        else:
            parser = self.parser
        columns: Union[List[Dict[str, Any]], None, Unset]
        if isinstance(self.columns, Unset):
            columns = UNSET
        elif isinstance(self.columns, list):
            columns = []
            for componentsschemasitem_columns_type_0_item_data in self.columns:
                componentsschemasitem_columns_type_0_item = (
                    componentsschemasitem_columns_type_0_item_data.to_dict()
                )
                columns.append(componentsschemasitem_columns_type_0_item)

        else:
            columns = self.columns
        dataview_definition = self.dataview_definition
        max_rows = self.max_rows

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "parser": parser,
            }
        )
        if columns is not UNSET:
            field_dict["columns"] = columns
        if dataview_definition is not UNSET:
            field_dict["dataview_definition"] = dataview_definition
        if max_rows is not UNSET:
            field_dict["max_rows"] = max_rows

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsTablesPreviewTableDataBody` from a dict"""
        d = src_dict.copy()

        def _parse_parser(data: object) -> Union["ItemParser", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasitem_parser_type_0 = ItemParser.from_dict(data)

                return componentsschemasitem_parser_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ItemParser", None], data)

        parser = _parse_parser(d.pop("parser"))

        def _parse_columns(data: object) -> Union[List["ItemColumn"], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasitem_columns_type_0 = []
                _componentsschemasitem_columns_type_0 = data
                for (
                    componentsschemasitem_columns_type_0_item_data
                ) in _componentsschemasitem_columns_type_0:
                    componentsschemasitem_columns_type_0_item = ItemColumn.from_dict(
                        componentsschemasitem_columns_type_0_item_data
                    )

                    componentsschemasitem_columns_type_0.append(
                        componentsschemasitem_columns_type_0_item
                    )

                return componentsschemasitem_columns_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["ItemColumn"], None, Unset], data)

        columns = _parse_columns(d.pop("columns", UNSET))

        dataview_definition = d.pop("dataview_definition", UNSET)

        max_rows = d.pop("max_rows", UNSET)

        collections_tables_preview_table_data_body = cls(
            parser=parser,
            columns=columns,
            dataview_definition=dataview_definition,
            max_rows=max_rows,
        )

        return collections_tables_preview_table_data_body



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.cdeset import Cdeset


T = TypeVar("T", bound="CdesetList")


@_attrs_define
class CdesetList:
    """CdesetList model

    Attributes:
        cdesets (List['Cdeset']):
    """

    cdesets: List["Cdeset"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        cdesets = []
        for cdesets_item_data in self.cdesets:
            cdesets_item = cdesets_item_data.to_dict()
            cdesets.append(cdesets_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "cdesets": cdesets,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CdesetList` from a dict"""
        d = src_dict.copy()
        cdesets = []
        _cdesets = d.pop("cdesets")
        for cdesets_item_data in _cdesets:
            cdesets_item = Cdeset.from_dict(cdesets_item_data)

            cdesets.append(cdesets_item)

        cdeset_list = cls(
            cdesets=cdesets,
        )

        return cdeset_list



import datetime
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.collection_stats_item_stats import CollectionStatsItemStats


T = TypeVar("T", bound="CollectionStats")


@_attrs_define
class CollectionStats:
    """CollectionStats model

    Attributes:
        item_stats (CollectionStatsItemStats):
        newest_item_date (datetime.datetime):
        oldest_item_date (datetime.datetime):
    """

    item_stats: "CollectionStatsItemStats"
    newest_item_date: datetime.datetime
    oldest_item_date: datetime.datetime

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        item_stats = self.item_stats.to_dict()
        newest_item_date = self.newest_item_date.isoformat()
        oldest_item_date = self.oldest_item_date.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "itemStats": item_stats,
                "newestItemDate": newest_item_date,
                "oldestItemDate": oldest_item_date,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionStats` from a dict"""
        d = src_dict.copy()
        item_stats = CollectionStatsItemStats.from_dict(d.pop("itemStats"))

        newest_item_date = isoparse(d.pop("newestItemDate"))

        oldest_item_date = isoparse(d.pop("oldestItemDate"))

        collection_stats = cls(
            item_stats=item_stats,
            newest_item_date=newest_item_date,
            oldest_item_date=oldest_item_date,
        )

        return collection_stats



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.highlights_metadata import HighlightsMetadata
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Highlights")


@_attrs_define
class Highlights:
    """Highlights model

    Attributes:
        metadata (HighlightsMetadata):
        name (List[str]): A sequence of strings. The odd-numbered zero-based indices
            should be highlighted.
        description (Union[Unset, List[str]]): A sequence of strings. The odd-numbered zero-based indices
            should be highlighted.
        tags (Union[Unset, List[List[str]]]):
    """

    metadata: "HighlightsMetadata"
    name: List[str]
    description: Union[Unset, List[str]] = UNSET
    tags: Union[Unset, List[List[str]]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        metadata = self.metadata.to_dict()
        name = self.name

        description: Union[Unset, List[str]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description

        tags: Union[Unset, List[List[str]]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = []
            for tags_item_data in self.tags:
                tags_item = tags_item_data

                tags.append(tags_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "metadata": metadata,
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Highlights` from a dict"""
        d = src_dict.copy()
        metadata = HighlightsMetadata.from_dict(d.pop("metadata"))

        name = cast(List[str], d.pop("name"))

        description = cast(List[str], d.pop("description", UNSET))

        tags = []
        _tags = d.pop("tags", UNSET)
        for tags_item_data in _tags or []:
            tags_item = cast(List[str], tags_item_data)

            tags.append(tags_item)

        highlights = cls(
            metadata=metadata,
            name=name,
            description=description,
            tags=tags,
        )

        return highlights



"""Initiate a multipart item data upload sequence"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.collections_items_initiate_multipart_upload_body import (
    CollectionsItemsInitiateMultipartUploadBody,
)
from ...models.multipart_upload_details import MultipartUploadDetails
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    body: CollectionsItemsInitiateMultipartUploadBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/collections/{collection_id}/{version_id}/items/{item_id}/multipartUpload/initiate".format(
            collection_id=collection_id,
            version_id=version_id,
            item_id=item_id,
        ),
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[MultipartUploadDetails, ServerError]:
    if response.status_code == HTTPStatus.OK:
        response_200 = MultipartUploadDetails.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ServerError.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[MultipartUploadDetails, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsInitiateMultipartUploadBody,
) -> Response[Union[MultipartUploadDetails, ServerError]]:
    """Initiate a multipart item data upload sequence

    Starts the process of uploading an item's data as a sequence
    of parts. This method will return a mapping of byte ranges to
    URLs; the client should break up the item data into segments
    and upload each segment to the respective URL. Uploads can be
    performed out-of-order and can be retried if an individual
    upload fails. Once all uploads are complete, the
    `complete_multipart_upload' method should be requested.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsInitiateMultipartUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[MultipartUploadDetails, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsInitiateMultipartUploadBody,
) -> Union[MultipartUploadDetails]:
    """Initiate a multipart item data upload sequence

    Starts the process of uploading an item's data as a sequence
    of parts. This method will return a mapping of byte ranges to
    URLs; the client should break up the item data into segments
    and upload each segment to the respective URL. Uploads can be
    performed out-of-order and can be retried if an individual
    upload fails. Once all uploads are complete, the
    `complete_multipart_upload' method should be requested.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsInitiateMultipartUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[MultipartUploadDetails]
    """

    response = sync_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsInitiateMultipartUploadBody,
) -> Response[Union[MultipartUploadDetails, ServerError]]:
    """Initiate a multipart item data upload sequence

    Starts the process of uploading an item's data as a sequence
    of parts. This method will return a mapping of byte ranges to
    URLs; the client should break up the item data into segments
    and upload each segment to the respective URL. Uploads can be
    performed out-of-order and can be retried if an individual
    upload fails. Once all uploads are complete, the
    `complete_multipart_upload' method should be requested.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsInitiateMultipartUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[MultipartUploadDetails, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsInitiateMultipartUploadBody,
) -> Union[MultipartUploadDetails]:
    """Initiate a multipart item data upload sequence

    Starts the process of uploading an item's data as a sequence
    of parts. This method will return a mapping of byte ranges to
    URLs; the client should break up the item data into segments
    and upload each segment to the respective URL. Uploads can be
    performed out-of-order and can be retried if an individual
    upload fails. Once all uploads are complete, the
    `complete_multipart_upload' method should be requested.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsInitiateMultipartUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[MultipartUploadDetails]
    """

    response = await asyncio_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



import datetime
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.account_metadata import AccountMetadata
from ..models.visibility_level import VisibilityLevel
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Account")


@_attrs_define
class Account:
    """Account model

    Attributes:
        created_on (datetime.datetime):
        id (str):
        is_personal (bool):
        metadata (AccountMetadata):
        name (str):
        members_group_id (Union[Unset, str]):
        visibility_level (Union[Unset, VisibilityLevel]):
    """

    created_on: datetime.datetime
    id: str
    is_personal: bool
    metadata: "AccountMetadata"
    name: str
    members_group_id: Union[Unset, str] = UNSET
    visibility_level: Union[Unset, VisibilityLevel] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        created_on = self.created_on.isoformat()
        id = self.id
        is_personal = self.is_personal
        metadata = self.metadata.to_dict()
        name = self.name
        members_group_id = self.members_group_id
        visibility_level: Union[Unset, str] = UNSET
        if not isinstance(self.visibility_level, Unset):
            visibility_level = self.visibility_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "createdOn": created_on,
                "id": id,
                "isPersonal": is_personal,
                "metadata": metadata,
                "name": name,
            }
        )
        if members_group_id is not UNSET:
            field_dict["membersGroupId"] = members_group_id
        if visibility_level is not UNSET:
            field_dict["visibilityLevel"] = visibility_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Account` from a dict"""
        d = src_dict.copy()
        created_on = isoparse(d.pop("createdOn"))

        id = d.pop("id")

        is_personal = d.pop("isPersonal")

        metadata = AccountMetadata.from_dict(d.pop("metadata"))

        name = d.pop("name")

        members_group_id = d.pop("membersGroupId", UNSET)

        _visibility_level = d.pop("visibilityLevel", UNSET)
        visibility_level: Union[Unset, VisibilityLevel]
        if isinstance(_visibility_level, Unset):
            visibility_level = UNSET
        else:
            visibility_level = VisibilityLevel(_visibility_level)

        account = cls(
            created_on=created_on,
            id=id,
            is_personal=is_personal,
            metadata=metadata,
            name=name,
            members_group_id=members_group_id,
            visibility_level=visibility_level,
        )

        return account



from enum import Enum


class ItemStatusValue(str, Enum):
    FAILED = "failed"
    NOT_APPLICABLE = "not-applicable"
    OK = "ok"
    PENDING = "pending"
    SERVER_ERROR = "server-error"
    WARNING = "warning"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AuthorizedActionsItem(str, Enum):
    CREATE = "Create"
    DELETE = "Delete"
    GET = "Get"
    GRANT = "Grant"
    UPDATE = "Update"
    VIEW = "View"

    def __str__(self) -> str:
        return str(self.value)



# mypy: disable-error-code="import-not-found, unused-ignore"
# %!pip install pydelfini
import pandas as pd
import pydelfini

client = pydelfini.login("delfini.local")


collection = client.get_collection_by_name("MHSVI")
# collection = pydelfini.explore()

dataframe = collection.get_table("my-folder/my-data")

folder_list = collection.folder("my-folder")
dataframe = folder_list.get_table("my-data")

for item in folder_list:
    # do something
    pass

for item in collection.walk():
    # recurse over all items in the collection
    pass

new_dataframe = pd.DataFrame({"A": [1, 2, 3]})
collection.write_table("output-data", new_dataframe, format="csv")


# more ideas
collection.new_folder("new")  # type: ignore

with collection.open("my-binary", "w") as fp:
    fp.write("a,b,c\n10,1,15\n20,2,25\n")

collection.set_parser(  # type: ignore
    "my-binary", name="csv", options={}, columns={"a": {"name": "a", "type": "Integer"}}
)



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="UserAdminUpdate")


@_attrs_define
class UserAdminUpdate:
    """UserAdminUpdate model

    Attributes:
        is_disabled (Union[Unset, bool]):
    """

    is_disabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        is_disabled = self.is_disabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_disabled is not UNSET:
            field_dict["is_disabled"] = is_disabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`UserAdminUpdate` from a dict"""
        d = src_dict.copy()
        is_disabled = d.pop("is_disabled", UNSET)

        user_admin_update = cls(
            is_disabled=is_disabled,
        )

        user_admin_update.additional_properties = d
        return user_admin_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ColumnSchemaType(str, Enum):
    BINARY = "Binary"
    BOOLEAN = "Boolean"
    CATEGORICAL = "Categorical"
    DATE = "Date"
    DATETIME = "Datetime"
    DECIMAL = "Decimal"
    DURATION = "Duration"
    FLOAT = "Float"
    INTEGER = "Integer"
    NULL = "Null"
    STRING = "String"
    TIME = "Time"
    UNKNOWN = "Unknown"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.system_configuration_cache_cache_type import (
    SystemConfigurationCacheCacheType,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SystemConfigurationCache")


@_attrs_define
class SystemConfigurationCache:
    """SystemConfigurationCache model

    Attributes:
        cache_default_timeout (Union[Unset, int]):  Default: 300.
        cache_type (Union[Unset, SystemConfigurationCacheCacheType]):
    """

    cache_default_timeout: Union[Unset, int] = 300
    cache_type: Union[Unset, SystemConfigurationCacheCacheType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        cache_default_timeout = self.cache_default_timeout
        cache_type: Union[Unset, str] = UNSET
        if not isinstance(self.cache_type, Unset):
            cache_type = self.cache_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cache_default_timeout is not UNSET:
            field_dict["cache_default_timeout"] = cache_default_timeout
        if cache_type is not UNSET:
            field_dict["cache_type"] = cache_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SystemConfigurationCache` from a dict"""
        d = src_dict.copy()
        cache_default_timeout = d.pop("cache_default_timeout", UNSET)

        _cache_type = d.pop("cache_type", UNSET)
        cache_type: Union[Unset, SystemConfigurationCacheCacheType]
        if isinstance(_cache_type, Unset):
            cache_type = UNSET
        else:
            cache_type = SystemConfigurationCacheCacheType(_cache_type)

        system_configuration_cache = cls(
            cache_default_timeout=cache_default_timeout,
            cache_type=cache_type,
        )

        system_configuration_cache.additional_properties = d
        return system_configuration_cache

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionsItemsCopyItemBody")


@_attrs_define
class CollectionsItemsCopyItemBody:
    """CollectionsItemsCopyItemBody model

    Attributes:
        source_url (str):  Example: delfini://example.com/collection/version/item-id.
        folder_id (Union[Unset, str]):  Example: ROOT.
        name (Union[Unset, str]):
    """

    source_url: str
    folder_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        source_url = self.source_url
        folder_id = self.folder_id
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "sourceUrl": source_url,
            }
        )
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsItemsCopyItemBody` from a dict"""
        d = src_dict.copy()
        source_url = d.pop("sourceUrl")

        folder_id = d.pop("folderId", UNSET)

        name = d.pop("name", UNSET)

        collections_items_copy_item_body = cls(
            source_url=source_url,
            folder_id=folder_id,
            name=name,
        )

        return collections_items_copy_item_body



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.cache_file_type import CacheFileType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CacheFile")


@_attrs_define
class CacheFile:
    """CacheFile model

    Attributes:
        base_dir (Union[Unset, str]):
        disk_free_min (Union[Unset, int]):  Default: 4294967296.
        size_max_cacheable (Union[Unset, int]):  Default: 134217728.
        type (Union[Unset, CacheFileType]):
    """

    base_dir: Union[Unset, str] = UNSET
    disk_free_min: Union[Unset, int] = 4294967296
    size_max_cacheable: Union[Unset, int] = 134217728
    type: Union[Unset, CacheFileType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        base_dir = self.base_dir
        disk_free_min = self.disk_free_min
        size_max_cacheable = self.size_max_cacheable
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if base_dir is not UNSET:
            field_dict["base_dir"] = base_dir
        if disk_free_min is not UNSET:
            field_dict["disk_free_min"] = disk_free_min
        if size_max_cacheable is not UNSET:
            field_dict["size_max_cacheable"] = size_max_cacheable
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CacheFile` from a dict"""
        d = src_dict.copy()
        base_dir = d.pop("base_dir", UNSET)

        disk_free_min = d.pop("disk_free_min", UNSET)

        size_max_cacheable = d.pop("size_max_cacheable", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, CacheFileType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = CacheFileType(_type)

        cache_file = cls(
            base_dir=base_dir,
            disk_free_min=disk_free_min,
            size_max_cacheable=size_max_cacheable,
            type=type,
        )

        cache_file.additional_properties = d
        return cache_file

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.cache_mem_type import CacheMemType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CacheMem")


@_attrs_define
class CacheMem:
    """CacheMem model

    Attributes:
        cache_max (Union[Unset, int]):  Default: 1073741824.
        type (Union[Unset, CacheMemType]):
    """

    cache_max: Union[Unset, int] = 1073741824
    type: Union[Unset, CacheMemType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        cache_max = self.cache_max
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cache_max is not UNSET:
            field_dict["cache_max"] = cache_max
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CacheMem` from a dict"""
        d = src_dict.copy()
        cache_max = d.pop("cache_max", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, CacheMemType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = CacheMemType(_type)

        cache_mem = cls(
            cache_max=cache_max,
            type=type,
        )

        cache_mem.additional_properties = d
        return cache_mem

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains functions to log in to a Delfini instance"""
import os
import time
from http.cookiejar import parse_ns_headers  # type: ignore
from typing import Any
from typing import Union

import attrs

from .api.auth import auth_authenticate_user
from .api.auth import auth_get_csrf
from .api.auth import auth_get_session
from .api.auth import auth_providers_req
from .client import AuthenticatedClient
from .client import Client
from .models import AuthAuthenticateUserAuthenticationRequest as AuthRequest
from .models import AuthGetSessionResponse200


ValidFilename = Union[str, bytes, os.PathLike[str], os.PathLike[bytes]]


class LoginError(Exception):
    pass


class Login:
    """Mid-level login routines.

    Users wanting to log in to a Delfini instance will typically use
    :py:func:`pydelfini.login`. The functions in this class are most
    useful if you are working on a specialized Delfini client that
    predominantly uses :py:mod:`pydelfini.delfini_core`, or if you are
    working with workflows that do not support interactive login.

    Typical usage::

        from pydelfini.delfini_core import Client, Login

        # get an unauthenticated client
        client = Client(base_url='https://delfini.bioteam.net/api/v1')

        # log in with password
        auth_client = Login(client).with_password(username, password)

        # or with session ID
        auth_client = Login(client).with_session_id(session_id)

    """

    def __init__(self, client: Client) -> None:
        """A new Login instance."""
        self.client = client

    def _authenticated_client(self, session_id: str) -> AuthenticatedClient:
        client_attrs: dict[str, Any] = {}
        for attrib in attrs.fields(Client):
            if attrib.alias and attrib.init:
                client_attrs[attrib.alias] = getattr(self.client, attrib.name)
        client_attrs["token"] = session_id

        return AuthenticatedClient(**client_attrs)

    def with_password(self, username: str, password: str) -> AuthenticatedClient:
        """Log in using a username and password.

        Args:
            username (str): the user's username
            password (str): the user's password

        Raises:
            LoginError: if the login could not be completed

        Returns:
            a new Client that has been successfully logged in
        """
        cookie: dict[str, str] = {}
        with self.client as client:
            providers = auth_providers_req.sync(client=client)
            if "credentials" not in providers:
                raise LoginError(
                    f"credentials login not supported at {client._base_url}"
                )

            token = auth_get_csrf.sync(client=client)
            request = AuthRequest(
                user_name=username,
                password=password,
                callback_url="http://null",
                csrf_token=token.csrf_token,
                json=True,
            )
            login_response = auth_authenticate_user.sync_detailed(
                client=client, body=request
            )
            if "set-cookie" in login_response.headers:
                parsed_cookie = parse_ns_headers(
                    [login_response.headers["set-cookie"]]
                )[0]
                for k, v in parsed_cookie:
                    if k == "session":
                        cookie[k] = v
            else:
                raise LoginError("login failed")

        return self._authenticated_client(list(cookie.values())[0])

    def with_session_id(
        self, session_id: str, wait: bool = False
    ) -> AuthenticatedClient:
        """Log in using an existing session ID.

        You can get a session ID token by calling
        :py:mod:`.api.auth.auth_new_session` and using the
        :py:attr:`~.models.session_token.SessionToken.session_id`
        attribute of the response. To activate the session, the user
        should visit ``/login/activate/<activation_code>`` where
        ``<activation_code>`` is the value of
        :py:attr:`~.models.session_token.SessionToken.activation_code`
        in the response.


        Args:
            session_id (str): an active session ID
            wait (bool): if True, wait for the session to be activated

        Raises:
            LoginError: if the session could not be verified

        Returns:
            a new Client that has been successfully logged in

        """

        logged_in_client = self._authenticated_client(session_id)

        with logged_in_client as client:
            while True:
                session = auth_get_session.sync(client=client)
                if isinstance(session, AuthGetSessionResponse200):
                    break
                elif not wait:
                    raise LoginError("session token could not be verified")

                time.sleep(2)

        return self._authenticated_client(session_id)

    def from_token_file(self, filename: ValidFilename) -> AuthenticatedClient:
        """Log in using a session ID saved in a token file.

        This is typically used to make CLI interactions smoother,
        since the session has a short lifespan. However, if you need
        to share a session among multiple processes or otherwise want
        to persist a session between invocations, you can use
        :py:func:`to_token_file` to save the session token from an
        instance of :py:class:`~.client.AuthenticatedClient`, then use
        this method to load the token file in a new process.


        Args:
            filename: path to token file

        Raises:
            LoginError: if the session could not be verified

        Returns:
            a new Client that has been successfully logged in

        """

        with open(filename) as fp:
            data = fp.readlines()
            if data:
                session_id = data[-1].strip()
            else:
                raise LoginError("session token could not be read")

        return self.with_session_id(session_id, wait=False)


def to_token_file(client: AuthenticatedClient, filename: ValidFilename) -> None:
    """Save a session token from an AuthenticatedClient to a file.

    Since session tokens have a short lifespan, this is mostly useful
    for CLI interactions or other situations where a session should
    persist across multiple processes. See
    :py:func:`Login.from_token_file` for how to use the token file
    created by this method.


    Args:
        client: a fully-logged-in client, typically from :py:class:`Login`
        filename: path to token file

    """
    try:
        # make sure the file is created with proper permissions
        orig_umask = os.umask(0o077)
        with open(filename, "w") as fp:
            fp.write(f"# {client._base_url}\n{client.token.split(';')[0]}\n")
    finally:
        os.umask(orig_umask)


def token_file_get_base_url(filename: ValidFilename) -> str:
    """Retrieve the base URL of the Delfini session recorded in the token file.

    Args:
        filename: path to token file

    Returns:
        the base URL

    """

    with open(filename) as fp:
        return fp.readlines()[0].lstrip("#").strip()



import datetime
from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.project_metadata import ProjectMetadata


T = TypeVar("T", bound="Project")


@_attrs_define
class Project:
    """Project model

    Attributes:
        account_id (str):
        collections (List[str]):
        created_on (datetime.datetime):
        description (str):
        id (str):
        metadata (ProjectMetadata):
        name (str):
    """

    account_id: str
    collections: List[str]
    created_on: datetime.datetime
    description: str
    id: str
    metadata: "ProjectMetadata"
    name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        account_id = self.account_id
        collections = self.collections

        created_on = self.created_on.isoformat()
        description = self.description
        id = self.id
        metadata = self.metadata.to_dict()
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "account_id": account_id,
                "collections": collections,
                "createdOn": created_on,
                "description": description,
                "id": id,
                "metadata": metadata,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Project` from a dict"""
        d = src_dict.copy()
        account_id = d.pop("account_id")

        collections = cast(List[str], d.pop("collections"))

        created_on = isoparse(d.pop("createdOn"))

        description = d.pop("description")

        id = d.pop("id")

        metadata = ProjectMetadata.from_dict(d.pop("metadata"))

        name = d.pop("name")

        project = cls(
            account_id=account_id,
            collections=collections,
            created_on=created_on,
            description=description,
            id=id,
            metadata=metadata,
            name=name,
        )

        return project



import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.error import Error
from ..models.item_status_value import ItemStatusValue


T = TypeVar("T", bound="ItemStatusDetail")


@_attrs_define
class ItemStatusDetail:
    """ItemStatusDetail model

    Attributes:
        errors (List['Error']):
        on (datetime.datetime):
        status (ItemStatusValue):
    """

    errors: List["Error"]
    on: datetime.datetime
    status: ItemStatusValue

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        errors = []
        for errors_item_data in self.errors:
            errors_item = errors_item_data.to_dict()
            errors.append(errors_item)

        on = self.on.isoformat()
        status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "errors": errors,
                "on": on,
                "status": status,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ItemStatusDetail` from a dict"""
        d = src_dict.copy()
        errors = []
        _errors = d.pop("errors")
        for errors_item_data in _errors:
            errors_item = Error.from_dict(errors_item_data)

            errors.append(errors_item)

        on = isoparse(d.pop("on"))

        status = ItemStatusValue(d.pop("status"))

        item_status_detail = cls(
            errors=errors,
            on=on,
            status=status,
        )

        return item_status_detail



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.account import Account
from ..models.pagination import Pagination


T = TypeVar("T", bound="AccountList")


@_attrs_define
class AccountList:
    """AccountList model

    Attributes:
        accounts (List['Account']):
        pagination (Pagination):
    """

    accounts: List["Account"]
    pagination: "Pagination"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        accounts = []
        for accounts_item_data in self.accounts:
            accounts_item = accounts_item_data.to_dict()
            accounts.append(accounts_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "accounts": accounts,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AccountList` from a dict"""
        d = src_dict.copy()
        accounts = []
        _accounts = d.pop("accounts")
        for accounts_item_data in _accounts:
            accounts_item = Account.from_dict(accounts_item_data)

            accounts.append(accounts_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        account_list = cls(
            accounts=accounts,
            pagination=pagination,
        )

        return account_list



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.metadata_metric_type import MetadataMetricType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="MetadataMetric")


@_attrs_define
class MetadataMetric:
    """MetadataMetric model

    Attributes:
        description (str):
        name (str):
        type (Union[Unset, MetadataMetricType]):  Default: MetadataMetricType.METADATA.
    """

    description: str
    name: str
    type: Union[Unset, MetadataMetricType] = MetadataMetricType.METADATA

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        description = self.description
        name = self.name
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "description": description,
                "name": name,
            }
        )
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetadataMetric` from a dict"""
        d = src_dict.copy()
        description = d.pop("description")

        name = d.pop("name")

        _type = d.pop("type", UNSET)
        type: Union[Unset, MetadataMetricType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = MetadataMetricType(_type)

        metadata_metric = cls(
            description=description,
            name=name,
            type=type,
        )

        return metadata_metric



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="MetricEvent")


@_attrs_define
class MetricEvent:
    """MetricEvent model

    Attributes:
        description (str):
        dimensions (List[str]):
        name (str):
        value_units (str):
    """

    description: str
    dimensions: List[str]
    name: str
    value_units: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        description = self.description
        dimensions = self.dimensions

        name = self.name
        value_units = self.value_units

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "description": description,
                "dimensions": dimensions,
                "name": name,
                "value_units": value_units,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`MetricEvent` from a dict"""
        d = src_dict.copy()
        description = d.pop("description")

        dimensions = cast(List[str], d.pop("dimensions"))

        name = d.pop("name")

        value_units = d.pop("value_units")

        metric_event = cls(
            description=description,
            dimensions=dimensions,
            name=name,
            value_units=value_units,
        )

        return metric_event
